@@ -1,4 +1,4 @@
-#' Spellchecker
+#' Spell checking
 #'
 #' @param filename Path to a LaTeX file to check.
 #' @param pre_release Should the document be assumed to be final?
@@ -7,14 +7,12 @@
 #' @param ignore.lines Integer vector of lines to ignore (due to possibly spurious errors).
 #' @param known.correct Character vector of patterns known to be correct (which will never be raised by this function).
 #' @param known.wrong Character vector of patterns known to be wrong.
-#' @param bib_files Bibliography files (containing possible clues to misspellings).
+#' @param bib_files Bibliography files (containing possible clues to misspellings). If supplied, and this function would otherwise throw an error, the \code{.bib} files are read and any author names that match the misspelled words are added to the dictionary.
 #' @param check_etcs If \code{TRUE}, stop if any variations of \code{etc}, \code{ie}, and \code{eg} are present. (If they are typed literally, they may be formatted inconsistently. Using a macro ensures they appear consistently.)
 #' @param dict_lang Passed to \code{hunspell::dictionary}.
+#' @param rstudio Use the RStudio API?
 #' @param .report_error A function to provide context to any errors.
-#' @return Called primarily for its side-effect. If the spell check fails, the line at which the first error was detected, with an error message. If the check suceeds, \code{NULL} invisibly.
-#' @importFrom hunspell hunspell
-#' @importFrom hunspell dictionary
-#' @import hunspell
+#' @return Called primarily for its side-effect. If the spell check fails, the line at which the first error was detected, with an error message. If the check succeeds, \code{NULL} invisibly.
 #' 
 #' @details Extends and enhances \code{hunspell}. The advantage of this function is that you can add directives 
 #' in the document itself. To add a word \code{foobaz} to the dictionary (so its presence does not throw an error), write
@@ -23,19 +21,25 @@
 #' directive \code{\% ignore_spelling_in: mycmd} which will ignore the spelling of words within the first argument
 #' of \code{\\mycmd}.
 #' 
-#' Other advantages include skipping the contents of certain commands, the spelling of which need not be checked 
-#' as they are not printed, \code{viz.} citation and cross-reference commands, and certain optional arguments.
+#' Another possible advantage is that only the root document need be supplied; 
+#' any files that are fed via \code{\\input} or \code{\\include} are checked (recursively).
+#' 
+#' Other advantages included skipping the contents of certain commands, the spelling of which need not be checked 
+#' as they are not printed, \code{viz.} citation and cross-reference commands, and certain optional arguments. These
+#' have been fixed by \code{hunspell} since the first version of this package.
 #' 
 #' The package comes with a suite of \code{\link{correctly_spelled_words}} that were not present in \code{hunspell}'s 
 #' dictionary.  
 #' 
-#' This function should be quite fast. I aim for less than 500 ms on a real-world report of around 100 pages.
-#' The function is slower when consulting \code{bib_files}, though I recommend adding authors, titles, etc. 
+#' This function should be quite fast, but slower than \code{hunspell::hunspell} (which it invokes). 
+#' I aim for less than 500 ms on a real-world report of around 100 pages.
+#' The function is slower when it needs to consult \code{bib_files}, though I recommend adding authors, titles, etc. 
+#' to the dictionary
 #' explicitly, or using \code{citeauthor} and friends. 
 #' 
 #' This function is forked from \url{https://github.com/hughparsonage/grattanReporter} to parse reports of the Grattan Institute, Melbourne for errors. See
 #' \url{https://github.com/HughParsonage/grattex/blob/master/doc/grattexDocumentation.pdf} for the full spec.
-#' Some checks have been omitted in this package.
+#' Some checks that package performs have been omitted in this package.
 #' 
 #' @examples 
 #' 
@@ -58,9 +62,14 @@ check_spelling <- function(filename,
                            bib_files,
                            check_etcs = TRUE,
                            dict_lang = "en_GB",
+                           rstudio = FALSE,
                            .report_error){
   if (missing(.report_error)){
-    .report_error <- function(...) report2console(...)
+    if (rstudio) {
+      .report_error <- function(...) report2console(file = filename, ..., rstudio = TRUE)
+    } else {
+      .report_error <- function(...) report2console(...)
+    }
   }
 
   file_path <- dirname(filename)
@@ -85,7 +94,7 @@ check_spelling <- function(filename,
     extra_known_wrong <-
       lines[grepl("^[%] stop_if_present[:]", lines, perl = TRUE)] %>%
       gsub("% stop_if_present: ", "", ., fixed = TRUE) %>%
-      trimws %>%
+      stri_trim_both %>%
       strsplit(split = " ", fixed = TRUE) %>%
       unlist
 
@@ -116,7 +125,7 @@ check_spelling <- function(filename,
     words_to_add <-
       lines[grepl("% add_to_dictionary: ", lines, fixed = TRUE)] %>%
       gsub("% add_to_dictionary: ", "", ., fixed = TRUE) %>%
-      trimws %>%
+      stri_trim_both %>%
       strsplit(split = " ", fixed = TRUE) %>%
       unlist
 
@@ -141,12 +150,12 @@ check_spelling <- function(filename,
   }
 
   # inputs and includes
-  inputs_in_doc <- length(grep("\\\\(?:(?:input)|(?:include(?!(graphics))))",
+  inputs_in_doc <- length(grep("\\\\(?:(?:input)|(?:include(?!(?:graphics|pdf))))",
                                lines_after_begin_document,
                                perl = TRUE))
 
   if (inputs_in_doc > 0){
-    inputs <- gsub("^\\\\(?:(?:input)|(?:include(?!(?:graphics))))[{](.*(?:\\.tex)?)[}]$",
+    inputs <- gsub("^\\\\(?:(?:input)|(?:include(?!(?:graphics|pdf))))[{](.*(?:\\.tex)?)[}]$",
                    "\\1",
                    lines_after_begin_document[grepl("^\\\\(?:(?:input)|(?:include(?!(?:graphics))))[{](.*(\\.tex)?)[}]$",
                                                     lines_after_begin_document,
@@ -167,7 +176,8 @@ check_spelling <- function(filename,
                                             paste0(input, ".tex")),
                        pre_release = pre_release,
                        known.correct = known.correct,
-                       known.wrong = known.wrong)
+                       known.wrong = known.wrong, 
+                       rstudio = rstudio)
       }
     }
   }
@@ -245,22 +255,9 @@ check_spelling <- function(filename,
   }
 
   # Ignore captionsetups
-  lines <- replace_LaTeX_argument(lines, "captionsetup", "")
+  lines <- replace_nth_LaTeX_argument(lines, "captionsetup", n = 1L, replacement = "")
 
   # Valid ordinal patterns are permitted
-  ordinal_pattern <-
-    paste0("((?<!1)1(\\\\textsuperscript\\{)?st)",
-           "|",
-           "((?<!1)2(\\\\textsuperscript\\{)?nd)",
-           "|",
-           "((?<!1)3(\\\\textsuperscript\\{)?rd)",
-           "|",
-           "(([04-9]|(1[1-3]))(\\\\textsuperscript\\{)?th)")
-  stopifnot(identical(grepl(ordinal_pattern,
-                            c("3rd", "11th", "21st", "13th", "13rd", "101st", "11st", "funding", "3\\textsuperscript{rd}"),
-                            perl = TRUE),
-                      c(TRUE, TRUE, TRUE, TRUE, FALSE, TRUE, FALSE, FALSE, TRUE)))
-
   lines <-
     gsub(ordinal_pattern,
          "",
@@ -270,14 +267,19 @@ check_spelling <- function(filename,
   lines <- remove_valid_contractions(lines)
 
   # Ignore phantoms
-  lines <- replace_LaTeX_argument(lines, command_name = "phantom", replacement = "PHANTOM")
-  lines <- replace_LaTeX_argument(lines, command_name = "gls", replacement = "ENTRY")
-  lines <- replace_LaTeX_argument(lines, command_name = "href", replacement = "correct")
+  lines <- replace_nth_LaTeX_argument(lines, command_name = "phantom", replacement = "PHANTOM")
+  lines <- replace_nth_LaTeX_argument(lines, command_name = "gls", replacement = "ENTRY")
+  lines <- replace_nth_LaTeX_argument(lines, command_name = "href", replacement = "correct")
   # Replace label argument in smallbox etc
   lines <- replace_nth_LaTeX_argument(lines,
                                       command_name = "begin.(?:(?:(?:very)?small)|(?:big))box[*]?[}]",
                                       n = 2L,
                                       replacement = "box:key")
+  lines <- replace_nth_LaTeX_argument(lines, command_name = "verysmallbox", optional = TRUE)
+  lines <- replace_nth_LaTeX_argument(lines,
+                                      command_name = "[CVcv]refrange",
+                                      n = 2L,
+                                      replacement = "second range key")
 
   ignore_spelling_in_line_no <-
     grep("^[%] ignore.spelling.in: ", lines, perl = TRUE)
@@ -295,7 +297,7 @@ check_spelling <- function(filename,
     commands_to_ignore <-
       lines[grepl("% ignore.spelling.in: ", lines, perl = TRUE)] %>%
       gsub("% ignore.spelling.in: ", "", ., perl = TRUE) %>%
-      trimws %>%
+      stri_trim_both %>%
       strsplit(split = " ", fixed = TRUE) %>%
       unlist
 
@@ -312,28 +314,6 @@ check_spelling <- function(filename,
   # Need to avoid optional arguments to commands: use the spaces?
   lines <- rm_editorial_square_brackets(lines)
 
-  lc_govt_pattern <-
-    paste0("(?:",
-           paste0("(?:Federal)",
-                  "|",
-                  "(?:Commonwealth)",
-                  "|",
-                  "(?:N(?:ew )?S(?:outh )?W(?:ales)?)",
-                  "|",
-                  "(?:Vic(?:torian?)?)",
-                  "|",
-                  "(?:Q(?:ueens)?l(?:an)?d)",
-                  "|",
-                  "(?:S(?:outh )?A(?:ustralian?)?)",
-                  "|",
-                  "(?:W(?:estern )?A(?:ustralian?)?)",
-                  "|",
-                  "(?:N(?:orthern? )?T(?:erritory)?)",
-                  "|",
-                  "(?:A(?:ustralian )?|C(?:apital )?T(?:erritory)?)"),
-           ") government",
-           "(?!\\s(?:schools?))")
-
   if (any(grepl(lc_govt_pattern, lines, perl = TRUE))){
     line_no <- grep(lc_govt_pattern, lines, perl = TRUE)[[1]]
     context <- lines[line_no]
@@ -343,7 +323,9 @@ check_spelling <- function(filename,
     stop("Wrong case: 'government' should start with uppercase G in this context.")
   }
 
-  if (any(grepl(sprintf("\\b(%s)\\b", wrongly_spelled_words), lines, perl = TRUE))){
+  # Only applicable to Grattan reports
+  if (AND(any(grepl("\\\\documentclass.*\\{grattan\\}", lines[1:10], perl = TRUE)),
+          any(grepl(sprintf("\\b(%s)\\b", wrongly_spelled_words), lines, perl = TRUE)))) {
     first_wrong_line_no <-
       grep(sprintf("\\b(%s)\\b", wrongly_spelled_words), lines, perl = TRUE) %>%
       .[1]
@@ -354,7 +336,8 @@ check_spelling <- function(filename,
            lines[first_wrong_line_no],
            perl = TRUE)
 
-    if (wrongly_spelled_word == "percent"){
+   
+    if (wrongly_spelled_word == "percent") {
       context <- paste0(lines[first_wrong_line_no], "\n",
                         "Use 'per cent', not 'percent'.")
     } else {
@@ -375,11 +358,15 @@ check_spelling <- function(filename,
   parsed <- hunspell(lines, format = "latex", dict = dictionary(dict_lang))
   all_bad_words <- unlist(parsed)
 
-  all_bad_words <- setdiff(all_bad_words,
-                           c(CORRECTLY_SPELLED_WORDS_CASE_SENSITIVE,
-                             correctly_spelled_words,
-                             words_to_add,
-                             known.correct))
+  # Faster than using hunspell(add_words = )
+  dictionary_additions <- c(CORRECTLY_SPELLED_WORDS_CASE_SENSITIVE,
+                            correctly_spelled_words,
+                            words_to_add,
+                            known.correct)
+  
+  all_bad_words <- 
+    all_bad_words[all_bad_words %notin% dictionary_additions] %>%
+    unique
 
   are_misspelt <- vapply(parsed, not_length0, logical(1))
 
@@ -403,9 +390,13 @@ check_spelling <- function(filename,
         unlist
 
       authors_in_bib_and_doc <-
-        intersect(grep("^[A-Z]", all_bad_words,
-                       value = TRUE, perl = TRUE),
+        intersect(grep("^[A-Z]",
+                       all_bad_words,
+                       value = TRUE,
+                       perl = TRUE),
                   authors_in_bib)
+      
+      dictionary_additions <- c(dictionary_additions, authors_in_bib_and_doc)
     } else {
       authors_in_bib <- authors_in_bib_and_doc <- NULL
     }
@@ -413,17 +404,7 @@ check_spelling <- function(filename,
 
     for (line_w_misspell in which(are_misspelt)){
       # If bad words %in% ... don't bother checking
-      bad_words <- setdiff(parsed[[line_w_misspell]],
-                           c(CORRECTLY_SPELLED_WORDS_CASE_SENSITIVE, correctly_spelled_words, words_to_add, known.correct))
-
-      if (!pre_release){
-        if (!is.null(authors_in_bib_and_doc)){
-          bad_words_no_proper_nouns <- setdiff(bad_words, authors_in_bib)
-          bad_words <- bad_words_no_proper_nouns
-        }
-      }
-
-      if (not_length0(bad_words)){
+      if (any(parsed[[line_w_misspell]] %notin% dictionary_additions)) {
         bad_line <- lines[[line_w_misspell]]
         # For timing
         bad_line_corrected <- gsub(gwp,
@@ -440,8 +421,11 @@ check_spelling <- function(filename,
                             format = "latex",
                             dict = dictionary("en_GB"))
 
-        if (not_length0(recheck[[1]])){
-          bad_word <- bad_words[[1]]
+        if (not_length0(recheck[[1]])) {
+          # We've discovered a likely misspelling.
+          # No need for performance, code within this if-statement
+          # is just to prepare the error message.
+          bad_word <- recheck[[1]][[1]]
           nchar_of_badword <- nchar(bad_word)
 
           chars_b4_badword <-
@@ -459,12 +443,21 @@ check_spelling <- function(filename,
             }
 
           .report_error(line_no = line_w_misspell,
+                        column = chars_b4_badword + nchar_of_badword + 1L,
                         context = context,
                         error_message = paste0("Spellcheck failed: '", bad_word, "'"),
                         extra_cat_post = c("\n",
                                            rep(" ", chars_b4_badword + 5 + nchar(line_w_misspell)),
                                            rep("^", nchar_of_badword),
                                            "\n"))
+          # Use %in% to avoid named logical var.
+          if (rstudio && Sys.info()['sysname'] %in% "Windows") {
+            suggested <- hunspell::hunspell_suggest(words = bad_word)[[1]]
+            # Use charToRaw to avoid a trailing newline.
+            # The extra space will be discarded (for some reason).
+            utils::writeClipboard(charToRaw(paste0(suggested[1], " ")))
+            cat("\nSuggested:\t", suggested)
+          }
           stop("Spellcheck failed on above line with '", bad_word, "'")
         }
       }
