@@ -1,11 +1,18 @@
 
-`%notin%` <- Negate(`%in%`)
-
 AND <- `&&`
 OR <- `||`
 
 not_length0 <- function(x) as.logical(length(x))
 
+lead <- function(x, n = 1L, default = NA) shift(x, type = "lead", n = n, fill = default)
+ lag <- function(x, n = 1L, default = NA) shift(x, type = "lag", n = n, fill = default)
+
+fill_blanks <- function(S) {
+   # from zoo
+   L <- !is.na(S)
+   c(S[L][1L], S[L])[cumsum(L) + 1L]
+}
+ 
 # takes a vector of froms and tos and takes their union
 seq.default.Vectorized <- function(x, y)
   Vectorize(seq.default, vectorize.args = c("from", "to"))(x, y)
@@ -77,8 +84,8 @@ nth_min.int <- function(x, n){
   sort.int(x)[n]
 }
 
-strip_comments <- function(lines){
-  gsub("(?<!(\\\\))[%].*$", "%", lines, perl = TRUE)
+strip_comments <- function(lines) {
+  sub("(?<!(\\\\))[%].*$", "%", lines, perl = TRUE)
 }
 
 move_to <- function(to.dir, from.dir = ".", pattern = "\\.((pdf)|(tex)|(cls)|(sty)|(Rnw)|(bib)|(png)|(jpg))$"){
@@ -89,8 +96,7 @@ move_to <- function(to.dir, from.dir = ".", pattern = "\\.((pdf)|(tex)|(cls)|(st
                   include.dirs = FALSE)
   x.dirs <- file.path(to.dir, 
                       list.dirs(path = from.dir, recursive = TRUE, full.names = TRUE))
-  dir_create <- function(x) if (!dir.exists(x)) dir.create(x)
-  lapply(x.dirs, dir_create)
+  lapply(x.dirs, hutils::provide.dir)
   file.copy(x, file.path(to.dir, x), overwrite = TRUE, recursive = FALSE)
   setwd(to.dir)
   cat("   Attempting compilation in temp directory:", to.dir, "\n")
@@ -102,4 +108,53 @@ r4 <- function(a, b, d, e) sprintf("%s%s%s%s", a, b, d, e)
 r5 <- function(a, b, d, e, f) sprintf("%s%s%s%s%s", a, b, d, e, f)
 r9 <- function(a1, a2, a3, a4, a5, a6, a7, a8, a9) sprintf("%s%s%s%s%s%s%s%s%s", a1, a2, a3, a4, a5, a6, a7, a8, a9)
 
-trimws_if_char <- function(x) if (is.character(x)) trimws(x) else x
+trimws_if_char <- function(x) if (is.character(x)) stri_trim_both(x) else x
+
+# for printing parsed lines
+print_transpose_data.table <- function(DT, file = "") {
+  cat <- function(...) base::cat(..., file = file, append = TRUE)
+  
+  max_nchar <- function(v) {
+    v[v == "\\"] <- "@"
+    v_na <- is.na(v)
+    out <- as.character(v)
+    out[v_na] <- ""
+    max(nchar(encodeString(out), type = "width"))
+  }
+  
+  char_width <- max(vapply(DT, max_nchar, integer(1)))
+  max_width_names <- max(nchar(names(DT)))
+  
+  for (var in names(DT)) {
+    cat(formatC(var, width = max_width_names), ":")
+    v <- DT[[var]]
+    v_na <- is.na(v)
+    v <- as.character(v)
+    v[v == "\\"] <- "@"
+    v[v_na] <- "."
+    v <- formatC(v, width = char_width)
+    # stop(var)
+    cat(v, sep = "")
+    cat("\n")
+  }
+  
+}
+
+return_first_nonNA <- function(x) {
+  if (anyNA(x)) {
+    not_nax <- !is.na(x)
+    if (any(not_nax)) {
+      out <- first(x[not_nax])
+    } else {
+      out <- x[1]
+    }
+  } else {
+    out <- x[1]
+  }
+  out
+}
+
+
+
+
+
